"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveAuthCookieName = exports.getProps = exports.DEFAULT_AUTH_COOKIE_NAME = void 0;
const hypercontroller_1 = require("@hyperstackjs/hypercontroller");
const hyperstack_1 = require("hyperstack");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('hyperstack:init-jwt');
exports.DEFAULT_AUTH_COOKIE_NAME = 'HS_AUTH';
const key = 'jwt';
const getProps = () => hyperstack_1.context.initializerProps(key);
exports.getProps = getProps;
const resolveAuthCookieName = (cookieNameConfig) => {
    if (!cookieNameConfig) {
        return null;
    }
    if (cookieNameConfig === true) {
        return exports.DEFAULT_AUTH_COOKIE_NAME;
    }
    else if (typeof cookieNameConfig == 'string' &&
        cookieNameConfig.length > 0) {
        return cookieNameConfig;
    }
    else {
        return null;
    }
};
exports.resolveAuthCookieName = resolveAuthCookieName;
exports.default = (createLoader) => (context) => __awaiter(void 0, void 0, void 0, function* () {
    const loader = createLoader(context);
    const config = context.config();
    const jwtSecretOrPublicKey = config.controllers.jwtSecret;
    if (!jwtSecretOrPublicKey) {
        throw new Error('No JWT secret is set in configuration');
    }
    const bearer = config.controllers.bearer || undefined;
    const resolvedAuthCookieName = (0, exports.resolveAuthCookieName)(config.controllers.authCookieName);
    let finalParams;
    const baseParams = {
        secret: jwtSecretOrPublicKey,
        loader,
        bearer,
        jwtOptions: {
            expiresIn: config.controllers.jwtExpiry || '14d',
            algorithm: config.controllers.jwtAlgorithm || 'HS512',
        },
    };
    if (resolvedAuthCookieName) {
        finalParams = Object.assign(Object.assign({}, baseParams), { authCookieName: resolvedAuthCookieName });
    }
    else {
        finalParams = Object.assign({}, baseParams);
    }
    const { MustAuthWithJWT, MustAuthRouteWithJWT, signJWT, verifyJWT } = (0, hypercontroller_1.authWithJWT)(finalParams);
    return {
        provideProps() {
            return {
                [key]: {
                    MustAuthWithJWT,
                    MustAuthRouteWithJWT,
                    signJWT,
                    verifyJWT,
                    currentUser(req) {
                        if (!req.user) {
                            debug('no user in request');
                            throw (0, hypercontroller_1.err)('no user');
                        }
                        return req.user;
                    },
                },
            };
        },
    };
});
